;;; File: er-macros-run.scm
(require 'er-macros 'contracts)
(import er-macros contracts)
(import-for-syntax (only contracts er-macro-rules))

(define-syntax-with-contract do-forever
  (er-macro-rules (%call/cc %lambda %let %loop)
     ((_ . body)
      `(,%call/cc (,%lambda (exit)
                    (,%let ,%loop ()
                      ,@body
                      (,%loop)))))))

(define-with-contract (foo n)
  (domain: (integer? n) (>= n 0))
  (range: (list? result))
  (let ((lst '()))
    (do-forever
      (if (zero? n)
        (exit lst))
        (begin
          (set! lst (cons 'a lst))
          (set! n (- n 1))))))

(er-macro-define-with-contract (my-or . args)
  "a variant of or"
  (with-renamed (%if %my-or)
    (if (null? args)
      #f
      (let ((tmp (car args)))
        `(,%if ,tmp ,tmp (,%my-or ,@(cdr args)))))))

(er-macro-define (freeze . xprs)
  (with-renamed (%lambda)
    `(,%lambda () ,@xprs)))

(define (f n) (+ n 10))

(define (bar)
  (er-macro-let (
    ((f n) (with-renamed ()  n))
    ((g n) (with-renamed (%f) `(,%f ,n)))
    )
    (list (f 1) (g 1))))

(define (baz)
  (er-macro-letrec (
    ((f n) (with-renamed ()  n))
    ((g n) (with-renamed (%f) `(,%f ,n)))
    )
    (list (f 1) (g 1))))

(define (run . xprs)
  (let loop ((xprs xprs))
    (if (null? xprs)
      (print "All tests passed!")
      (if (car xprs)
        (loop (cdr xprs))
        (error 'run "#### Some test failed! ####")))))

(run
  (equal? (with-aliases (identity %x %y) (list %x %y)) '(x y))
  (= (my-or #f #f 2 #f) 2)
  (= ((freeze 1 2 3)) 3)
  (eq? (my-or) #f)
  (equal? (foo 3) '(a a a))
  (equal? (bar) '(1 11))
  (equal? (baz) '(1 1))
  )
